#include "sparrowshell.h"

//#include "message.h"
#include "input.h"
#include "scene/scenetree.h"
#include "scene/meshnode.h"
#include "scene/textnode.h"
#include "mesh.h"
#include "phongmaterial.h"
#include "tools/utils.h"
#include "tools/font.h"

const unsigned int SparrowShell::BUFFER_MAX_LENGTH = 50;
const unsigned int SparrowShell::BUFFER_DISPLAYED_NUMBER = 10;
const unsigned int SparrowShell::SCROLLBAR_PIXEL_WIDTH = 10;
const float SparrowShell::SHELL_DEPTH = 10;

SparrowShell::SparrowShell(sf::Window* window, Input* input):
    m_window(window),
    m_input(input),
    m_position(glm::ivec2(0,0)),
    m_buffer(new ShellBuffer(BUFFER_MAX_LENGTH))
{
    sf::Vector2u size = m_window->getSize();
    m_dimension = glm::ivec2(size.x,size.y/2);
    Mesh* mesh = new Mesh();
    m_buffer->setFontSize(16.f);
    mesh->addRectangle2D(m_position,m_dimension);
    PhongMaterial *mat = new PhongMaterial();
    mat->diffuse = glm::vec3(0.1,0.1,0.1);
    mat->m_opacity = 0.5;
    mesh->setMaterial(mat);
    mesh->setDepth(SHELL_DEPTH);
    mesh->initGL();
    m_background = new MeshNode(mesh,false);
    this->addChild(m_background);
    this->addChild(m_buffer);
    m_scrollbar = ShellScrollBar(this);
}

void SparrowShell::out(std::string s)
{
    Font *shellfont = RESOURCE_GET(Font,"shellfont");
    TextNode* tnode = shellfont->getTextNode(s,glm::vec3(0.7,1,0.3),m_buffer->getFontSize());
    tnode->setDepth(SHELL_DEPTH+1);
    m_buffer->push(tnode);
    if(m_shellEnabled)
        m_scene->addToIndex(tnode);
    if (m_buffer->size() > SparrowShell::BUFFER_DISPLAYED_NUMBER)
        m_resizeBuffer = true;
}

void SparrowShell::scrollUp(){
    if (m_index + SparrowShell::BUFFER_DISPLAYED_NUMBER < m_buffer->size()){
        m_index++;
        m_indexMoved = true;
    }
}

void SparrowShell::scrollDown(){
    if (m_index > 0){
        m_index--;
        m_indexMoved = true;
    }
}

void SparrowShell::toggleShell(){
    m_shellEnabled = !m_shellEnabled;
//    for(auto child : m_children){
//        MeshNode* meshchild = dynamic_cast<MeshNode*>(child);
//    //    if(meshchild)
//  //          meshchild->toggleVisibility();
//    }
////    m_buffer->toggleBuffer();
////    m_background->toggleVisibility();
}

void SparrowShell::update(){
    for(auto action : m_input->getActions()){
        if(action == 15){
            toggleShell();
        }
    }
    m_buffer->update();
    m_scrollbar.update();
}
